function [im, dims] = datread(filename, nrows, ncols, nsets, datatype)
%% function [im dims] = datread(filename, nrows, ncols, nsets, datatype)
%% Requires: Image processing toolbox.
%%           Currently: reads only 1 channel imagesX
%%           datatype is optional (default is 'uchar')
%%             other possible values those supported by fread,eg. 'short'
%% Read a datfile, returning the result in im, and the associated dimensions
%%      in dims.
%% Currently: Requires that dimensions are specified,
%%            Reads only 1 channel images
%% To display, must normalize all values between 0 and 1.
%% Example:
%%   imagesc(imriv)
%%   montage(imrivseq/255, [115 170 5])
%%   truesize


error(nargchk(4, 5, nargin));
if nargin == 4
  datatype = 'uchar'
end

fid = fopen([filename,'/data'], 'r');
if fid == -1
  error(['Could not open file: ' filename])
end

% Preallocate memory
im = zeros(nrows, ncols*nsets);

[tmp_im, noRead] = fread(fid, [ncols, nrows], datatype);
if noRead ~= ncols*nrows 
    error(['While reading frame ', int2str(i), ' could only read ', int2str(noRead), ' bytes out of ', int2str(ncols*nrows)]);
end

im(imslice([nrows ncols nsets], 1)) = tmp_im';

for i = 2:nsets
  [tmp_im, noRead] = fread(fid, [ncols, nrows], datatype);
  if noRead ~= ncols*nrows 
    error(['While reading frame ', int2str(i), ' could only read ', int2str(noRead), ' bytes out of ', int2str(ncols*nrows)]);
    end
  im(imslice([nrows ncols nsets], i)) = tmp_im';
end  

fclose(fid);  
dims = [nrows ncols nsets];

