function datwrite(dirname, data, dims, datatype, comment)
% function datwrite(dirname, data, dims, datatype, comment)
% Requires: dims is a 3 element vector or [] or optional argument.
% Effects:  Writes data to dirname in DAT file format. 
%           If data is an image sequence in image deck format, specify dims
%           as [no_rows no_cols no_frames].
%           Otherwise, if data is a regular Matlab matrix, set dims = [],
%           and the default matlab matrix size will be used. 
%           format is as in dread, and defaults to double.
%           datatype: refer to fread precision argument
%           comment is written as 'created by dat'

error(nargchk(2,5,nargin));

if nargin < 5,  comment = ''; 
end
  
if nargin < 4,  datatype = 'double'; end
if nargin < 3,  dims = [size(data) 1]; end;

if dims == [],  dims = [size(data) 1]; end;

if prod(dims) ~= prod(size(data))
  fprintf(1, 'WARNING: prod(dims) ~= prod(size(data)). Proceeding...');
end

matlab_types = [ 
 'char          '
 'char*1        '
 'uchar         '
 'unsigned char '
 'schar         '
 'signed char   '
 'short         '
 'ushort        '
 'unsigned short'
 'int           '
 'uint          '
 'unsigned int  '
 'long          '
 'ulong         '
 'unsigned long '
 'float         '
 'float32       '
 'real*4        '
 'double        '
 'float64       '
 'real*8        '  
 'int8          '
 'integer*1     '
 'int16         '
 'integer*2     '
 'int32         '
 'integer*4     '
];

dat_types = [
    'unsigned_1'  
    'unsigned_1'  
    'unsigned_1'  
    'unsigned_1'  
    'signed_1  '  
    'signed_1  '  
    'signed_2  '  
    'unsigned_2'  
    'unsigned_2'  
    'signed_4  '  
    'unsigned_4'  
    'unsigned_4'  
    'signed_4  '  
    'unsigned_4'  
    'unsigned_4'  
    'float     '  
    'float     '  
    'float     '  
    'double    '  
    'double    '  
    'double    '  
    'signed_1  '  
    'signed_1  '  
    'signed_2  '  
    'signed_2  '  
    'signed_4  '  
    'signed_4  '];
 
if strcmp(datatype, 'integer*8')
  error('integer*8 not supported by DAT')
end

if strcmp(datatype, 'char') | strcmp(datatype, 'char*8')
  datatype = 'uchar';   %  ensure it is unsigned 
  fprintf(1, 'Warning: assuming char is UNSIGNED character');
end

outdatatype = translate_str(matlab_types, dat_types, datatype);

unix(['mkdir ' dirname]);

[dimrows, dimcols] = size(dims);

fid = ms_fopen([dirname, '/data'], 'wb');

if dimcols == 3
  % Image deck.  Write each image separately.
  for i=1:dims(3)
    fwrite(fid, data(imslice(dims, i))', datatype);
  end

  datdescriptor_write(dirname, dims, outdatatype, comment);

else
  % Regular Matlab matrix
  fwrite(fid, data', datatype);
  datdescriptor_write(dirname, dims, outdatatype, comment);
end


fclose(fid);
